@@ -42,6 +42,8 @@
 #include <gconf/gconf-client.h>
 
 #include <games-stock.h>
+#include <games-scores.h>
+#include <games-scores-dialog.h>
 
 #include "yahtzee.h"
 #include "gyahtzee.h"
@@ -50,7 +52,6 @@
 
 static char *appID = "gtali";
 static char *appName = N_("Tali");
-static guint lastHighScore = 0;
 static guint last_timeout = 0;
 static gboolean ready_to_advance_player;
 
@@ -61,7 +62,9 @@ static gboolean ready_to_advance_player;
 #endif
 
 #define NUMBER_OF_PIXMAPS    7
+#define GAME_TYPES 2
 #define DIE_SELECTED_PIXMAP  (NUMBER_OF_PIXMAPS-1)
+#define SCORES_CATEGORY (game_type == GAME_KISMET ? "Colors" : NULL)
 
 static char *dicefiles[NUMBER_OF_PIXMAPS] = { PP "gnome-dice-1.svg",
   PP "gnome-dice-2.svg",
@@ -72,7 +75,16 @@ static char *dicefiles[NUMBER_OF_PIXMAPS] = { PP "gnome-dice-1.svg",
   PP "gnome-dice-none.svg"
 };
 
-static GtkWidget *dicePixmaps[NUMBER_OF_DICE][NUMBER_OF_PIXMAPS];
+static char *kdicefiles[NUMBER_OF_PIXMAPS] = { PP "kismet1.svg",
+  PP "kismet2.svg",
+  PP "kismet3.svg",
+  PP "kismet4.svg",
+  PP "kismet5.svg",
+  PP "kismet6.svg",
+  PP "kismet-none.svg"
+};
+
+static GtkWidget *dicePixmaps[NUMBER_OF_DICE][NUMBER_OF_PIXMAPS][GAME_TYPES];
 
 GtkWidget *window;
 GtkWidget *ScoreList;
@@ -81,6 +93,7 @@ static GtkToolItem *diceBox[NUMBER_OF_DICE];
 static GtkWidget *rollLabel;
 static GtkWidget *mbutton;
 static GtkAction *scores_action;
+static gchar *game_type_string = NULL;
 
 const static GOptionEntry yahtzee_options[] = {
   {"delay", 'd', 0, G_OPTION_ARG_NONE, &DoDelay,
@@ -91,9 +104,25 @@ const static GOptionEntry yahtzee_options[] = {
    N_("Number of computer opponents"), N_("NUMBER")},
   {"humans", 'p', 0, G_OPTION_ARG_INT, &NumberOfHumans,
    N_("Number of human opponents"), N_("NUMBER")},
+  {"game", 'g', 0, G_OPTION_ARG_STRING, &game_type_string,
+   N_("Game choice: Regular or Colors"), N_("STRING")},
   {NULL}
 };
 
+static const GamesScoresCategory category_array[] = {
+    {"Regular", "Regular"}, {"Colors", "Colors"}, GAMES_SCORES_LAST_CATEGORY };
+
+static const GamesScoresDescription scoredesc = { 
+  category_array,
+  "Regular",
+  "gtali",
+  GAMES_SCORES_STYLE_PLAIN_DESCENDING
+};
+
+GamesScores *highscores;
+
+static GtkWidget *dialog = NULL;
+
 static gint gnome_modify_dice (GtkWidget * widget, gpointer data);
 static gint gnome_roll_dice (GtkWidget * widget, GdkEvent * event,
 			     gpointer data);
@@ -110,9 +139,21 @@ update_roll_button_sensitivity (void)
     state |=
       gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (diceBox[i]));
 
+  if(!state){
+    gtk_button_set_label (GTK_BUTTON (mbutton), _("Roll all!"));
+    state = TRUE;
+  } else {
+    gtk_button_set_label (GTK_BUTTON (mbutton), _("Roll!"));
+    state = TRUE;
+  }
+
   state &= NumberOfRolls < 3;
   state &= !players[CurrentPlayer].comp;
 
+  if(GameIsOver ()){
+    state = FALSE;
+  }
+
   gtk_widget_set_sensitive (GTK_WIDGET (mbutton), state);
 }
 
@@ -121,11 +162,15 @@ CheerWinner (void)
 {
   int winner;
   int i;
+  GamesScoreValue score;
+  gint pos;
+  gchar *message;
 
   ShowoffPlayer (ScoreList, CurrentPlayer, 0);
 
   winner = FindWinner ();
 
+  /* draw. The score is returned as a negative value */
   if (winner < 0) {
     for (i = 0; i < NumberOfPlayers; i++) {
       if (total_score (i) == -winner) {
@@ -140,10 +185,29 @@ CheerWinner (void)
   ShowoffPlayer (ScoreList, winner, 1);
 
   if (winner < NumberOfHumans) {
-    lastHighScore = gnome_score_log ((guint) WinningScore, NULL, TRUE);
+    score.plain = (guint32) WinningScore;  
+
+    pos = games_scores_add_score (highscores, score);
+
+    if (pos > 0) {
+      if (dialog) {
+        gtk_window_present (GTK_WINDOW (dialog));
+      } else {
+        dialog = games_scores_dialog_new (highscores, _("Tali Scores"));
+        message =
+  	  g_strdup_printf ("<b>%s</b>\n\n%s", _("Congratulations!"),
+			 _("Your score has made the top ten."));
+        games_scores_dialog_set_message (GAMES_SCORES_DIALOG (dialog), message);
+        g_free (message);
+      }
+      games_scores_dialog_set_hilight (GAMES_SCORES_DIALOG (dialog), pos);
+
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_hide (dialog);
+    }
+
     update_score_state ();
-    if (lastHighScore)
-      ShowHighScores ();
+
   }
 
   if (players[winner].name)
@@ -154,7 +218,6 @@ CheerWinner (void)
     say (_("Game over!"));
 
 }
-
 static gboolean
 do_computer_turns (void)
 {
@@ -196,6 +259,7 @@ NextPlayer (void)
       g_source_remove (last_timeout);
       last_timeout = 0;
     }
+    update_roll_button_sensitivity ();
     CheerWinner ();
     return;
   }
@@ -267,6 +331,12 @@ ShowPlayer (int num, int field)
 
   if (upper_tot >= 63) {
     bonus = 35;
+    if (game_type == GAME_KISMET) {
+      if (upper_tot >= 78)
+        bonus = 75;
+      else if (upper_tot >= 71)
+        bonus = 55;
+    }
     upper_tot += bonus;
   }
 
@@ -309,6 +379,8 @@ GyahtzeeNewGame (void)
 
   say (_("Select dice to roll or choose a score slot."));
 
+  game_type = get_new_game_type();
+  games_scores_set_category(highscores, SCORES_CATEGORY);
   NewGame ();
   setup_score_list (ScoreList);
   UpdateRollLabel ();
@@ -341,25 +413,27 @@ UpdateRollLabel (void)
 }
 
 static void
-UpdateDiePixmap (int n)
+UpdateDiePixmap (int n, int prev_game_type)
 {
   static int last_val[NUMBER_OF_DICE] = { 0 };
 
-  gtk_widget_hide (dicePixmaps[n][last_val[n]]);
+  gtk_widget_hide (dicePixmaps[n][last_val[n]][prev_game_type]);
 
   last_val[n] = DiceValues[n].sel ? DIE_SELECTED_PIXMAP :
     DiceValues[n].val - 1;
-  gtk_widget_show (dicePixmaps[n][last_val[n]]);
+  gtk_widget_show (dicePixmaps[n][last_val[n]][game_type]);
 }
 
 void
 UpdateAllDicePixmaps (void)
 {
   int i;
+  static int prev_game_type = 0;
 
   for (i = 0; i < NUMBER_OF_DICE; i++) {
-    UpdateDiePixmap (i);
+    UpdateDiePixmap (i, prev_game_type);
   }
+  prev_game_type = game_type;
 }
 
 void
@@ -452,7 +526,7 @@ about_cb (GtkAction * action, gpointer data)
   gtk_show_about_dialog (GTK_WINDOW (window),
 			 "name", appName,
 			 "version", VERSION,
-			 "copyright", "Copyright \xc2\xa9 1998-2006 "
+			 "copyright", "Copyright \xc2\xa9 1998-2007 "
 			 "Free Software Foundation, Inc.",
 			 "license", license,
 			 "comments", _("A variation on poker with "
@@ -472,17 +546,11 @@ about_cb (GtkAction * action, gpointer data)
 void
 ShowHighScores (void)
 {
-  GtkWidget *dialog;
+  if (!dialog)
+    dialog = games_scores_dialog_new (highscores, _("Tali Scores"));
 
-  dialog = gnome_scores_display (appName, appID, NULL, lastHighScore);
-  if (dialog != NULL) {
-    gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
-    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
-    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  }
+  gtk_dialog_run (GTK_DIALOG (dialog));    
+  gtk_widget_hide (dialog);
 }
 
 static gint
@@ -502,8 +570,10 @@ LoadDicePixmaps (void)
      * there is no way for us to copy a general GtkImage (the old 
      * code had a way for static images). */
     if (g_file_test (dicefiles[i], G_FILE_TEST_EXISTS))
-      for (j = 0; j < NUMBER_OF_DICE; j++)
-	dicePixmaps[j][i] = gtk_image_new_from_file (dicefiles[i]);
+      for (j = 0; j < NUMBER_OF_DICE; j++) {
+        dicePixmaps[j][i][GAME_YAHTZEE] = gtk_image_new_from_file(dicefiles[i]);
+        dicePixmaps[j][i][GAME_KISMET] = gtk_image_new_from_file(kdicefiles[i]);
+      }
     /* FIXME: What happens if the file isn't found. */
   }
 }
@@ -511,20 +581,10 @@ LoadDicePixmaps (void)
 void
 update_score_state (void)
 {
-  gchar **names = NULL;
-  gfloat *scores = NULL;
-  time_t *scoretimes = NULL;
-  gint top;
+  GList *top;
 
-  top = gnome_score_get_notable (appID, NULL, &names, &scores, &scoretimes);
-  if (top > 0) {
-    gtk_action_set_sensitive (scores_action, TRUE);
-    g_strfreev (names);
-    g_free (scores);
-    g_free (scoretimes);
-  } else {
-    gtk_action_set_sensitive (scores_action, FALSE);
-  }
+  top = games_scores_get (highscores);
+  gtk_action_set_sensitive (scores_action, top != NULL);
 }
 
 static void
@@ -658,8 +718,10 @@ GyahtzeeCreateMainWindow (void)
   for (i = 0; i < NUMBER_OF_DICE; i++) {
     tmp = gtk_vbox_new (FALSE, 0);
 
-    for (j = 0; j < NUMBER_OF_PIXMAPS; j++)
-      gtk_box_pack_start (GTK_BOX (tmp), dicePixmaps[i][j], FALSE, FALSE, 0);
+    for (j = 0; j < NUMBER_OF_PIXMAPS; j++) {
+      gtk_box_pack_start (GTK_BOX (tmp), dicePixmaps[i][j][GAME_YAHTZEE], FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (tmp), dicePixmaps[i][j][GAME_KISMET], FALSE, FALSE, 0);
+    }
 
     diceBox[i] = gtk_toggle_tool_button_new ();
     gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (diceBox[i]), tmp);
@@ -671,7 +733,7 @@ GyahtzeeCreateMainWindow (void)
 
     gtk_widget_show (GTK_WIDGET (diceBox[i]));
     gtk_widget_show (tmp);
-    gtk_widget_show (dicePixmaps[i][0]);
+  /*gtk_widget_show (dicePixmaps[i][0][game_type]);*/
   }
   gtk_widget_show (toolbar);
 
@@ -698,7 +760,7 @@ main (int argc, char *argv[])
   gint i;
   GOptionContext *context;
 
-  gnome_score_init (appID);
+  setgid_io_init ();
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
@@ -717,6 +779,8 @@ main (int argc, char *argv[])
 				GNOME_PARAM_GOPTION_CONTEXT, context,
 				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
+  highscores = games_scores_new (&scoredesc);
+
   gtk_window_set_default_icon_name ("gnome-tali");
 
   client = gconf_client_get_default ();
@@ -730,8 +794,8 @@ main (int argc, char *argv[])
 					   "/apps/gtali/NumberOfHumanOpponents",
 					   NULL);
 
-  if (NumberOfHumans < 0)
-    NumberOfHumans = 0;
+  if (NumberOfHumans < 1)
+    NumberOfHumans = 1;
   if (NumberOfComputers < 0)
     NumberOfComputers = 0;
 
@@ -740,6 +804,13 @@ main (int argc, char *argv[])
   if ((NumberOfHumans + NumberOfComputers) > MAX_NUMBER_OF_PLAYERS)
     NumberOfComputers = MAX_NUMBER_OF_PLAYERS - NumberOfHumans;
 
+  if (game_type_string) 
+    game_type = game_type_from_string(game_type_string);
+  else
+    game_type = game_type_from_string(gconf_client_get_string(client,
+                                      "/apps/gtali/GameType", NULL));
+  set_new_game_type(game_type);
+
   /* We ignore errors for these, they're boolean so it will be valid 
    * data even if not the right data and there's nothing else we
    * can do (if it's a systematic gconf problem, the calls above
